@@ -108,7 +108,7 @@ class KazooClient(object):
         are in seconds.
 
         :param hosts: Comma-separated list of hosts to connect to
-                      (e.g. 127.0.0.1:2181,127.0.0.1:2182).
+                      (e.g. 127.0.0.1:2181,127.0.0.1:2182,[::1]:2183).
         :param timeout: The longest to wait for a Zookeeper connection.
         :param client_id: A Zookeeper client id, used when
                           re-establishing a prior session connection.
@@ -218,7 +218,7 @@ class KazooClient(object):
 
         if command_retry is not None:
             self.retry = command_retry
-            if self.handler.sleep_func != self.comand_retry.sleep_func:
+            if self.handler.sleep_func != self.retry.sleep_func:
                 raise ConfigurationError("Command retry handler and event handler "
                                          " must use the same sleep func")
 
@@ -559,10 +559,9 @@ class KazooClient(object):
         if not self._live.is_set():
             raise ConnectionLoss("No connection to server")
 
-        sock = self.handler.socket()
-        sock.settimeout(self._session_timeout)
         peer = self._connection._socket.getpeername()
-        sock.connect(peer)
+        sock = self.handler.create_connection(
+            peer, timeout=self._session_timeout / 1000.0)
         sock.sendall(cmd)
         result = sock.recv(8192)
         sock.close()
