@@ -16,7 +16,7 @@ from gevent.queue import Queue
 from gevent import socket
 from zope.interface import implementer
 
-from kazoo.handlers.utils import create_tcp_socket
+from kazoo.handlers.utils import create_tcp_socket, create_tcp_connection
 from kazoo.interfaces import IAsyncResult
 from kazoo.interfaces import IHandler
 
@@ -119,6 +119,9 @@ class SequentialGeventHandler(object):
     def socket(self, *args, **kwargs):
         return create_tcp_socket(socket)
 
+    def create_connection(self, *args, **kwargs):
+        return create_tcp_connection(socket, *args, **kwargs)
+
     def event_object(self):
         """Create an appropriate Event object"""
         return gevent.event.Event()
