@@ -124,6 +124,9 @@ class LockingQueue(BaseQueue):
     connection status with Zookeeper or call :meth:`LockingQueue.holds_lock`
     method that will check if a node still holds the lock.
 
+    .. note::
+        :class:`LockingQueue` requires ZooKeeper 3.4 or above, since it is
+        using transactions.
     """
     lock = "/taken"
     entries = "/entries"
