@@ -5,6 +5,7 @@ import uuid
 from nose.tools import eq_
 from nose.tools import raises
 
+from kazoo.protocol.states import EventType
 from kazoo.testing import KazooTestCase
 
 
@@ -18,76 +19,68 @@ class KazooDataWatcherTests(KazooTestCase):
         update = threading.Event()
         data = [True]
 
+        # Make it a non-existent path
+        self.path += 'f'
+
         @self.client.DataWatch(self.path)
         def changed(d, stat):
             data.pop()
             data.append(d)
             update.set()
 
-        update.wait()
-        eq_(data, [b""])
-        update.clear()
-
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], b'fred')
-        update.clear()
-
-    def test_data_watcher2(self):
-        update = threading.Event()
-        data = [True]
-
-        @self.client.DataWatch(self.path, allow_missing_node=True)
-        def changed(d, stat):
-            data.pop()
-            data.append(d)
-            update.set()
-
-        update.wait()
+        update.wait(10)
         eq_(data, [None])
         update.clear()
 
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], None)
+        self.client.create(self.path, b'fred')
+        update.wait(10)
+        eq_(data[0], b'fred')
+        update.clear()
+
+    def test_data_watcher_with_event(self):
+        # Test that the data watcher gets passed the event, if it
+        # accepts three arguments
+        update = threading.Event()
+        data = [True]
+
+        # Make it a non-existent path
+        self.path += 'f'
+
+        @self.client.DataWatch(self.path)
+        def changed(d, stat, event):
+            data.pop()
+            data.append(event)
+            update.set()
+
+        update.wait(10)
+        eq_(data, [None])
+        update.clear()
+
+        self.client.create(self.path, b'fred')
+        update.wait(10)
+        eq_(data[0].type, EventType.CREATED)
         update.clear()
 
     def test_func_style_data_watch(self):
         update = threading.Event()
         data = [True]
 
-        def changed(d, stat):
-            data.pop()
-            data.append(d)
-            update.set()
-        self.client.DataWatch(self.path, changed)
-
-        update.wait()
-        eq_(data, [b""])
-        update.clear()
-
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], b'fred')
-        update.clear()
-
-    def test_func_style_data_watch2(self):
-        update = threading.Event()
-        data = [True]
+        # Make it a non-existent path
+        path = self.path + 'f'
 
         def changed(d, stat):
             data.pop()
             data.append(d)
             update.set()
-        self.client.DataWatch(self.path, changed, allow_missing_node=True)
+        self.client.DataWatch(path, changed)
 
-        update.wait()
+        update.wait(10)
         eq_(data, [None])
         update.clear()
 
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], None)
+        self.client.create(path, b'fred')
+        update.wait(10)
+        eq_(data[0], b'fred')
         update.clear()
 
     def test_datawatch_across_session_expire(self):
@@ -100,40 +93,21 @@ class KazooDataWatcherTests(KazooTestCase):
             data.append(d)
             update.set()
 
-        update.wait()
+        update.wait(10)
         eq_(data, [b""])
         update.clear()
 
         self.expire_session()
-        eq_(update.is_set(), False)
         self.client.retry(self.client.set, self.path, b'fred')
-        update.wait()
+        update.wait(25)
         eq_(data[0], b'fred')
 
-    def test_datawatch_across_session_expire2(self):
-        update = threading.Event()
-        data = [True]
-
-        @self.client.DataWatch(self.path, allow_missing_node=True)
-        def changed(d, stat):
-            data.pop()
-            data.append(d)
-            update.set()
-
-        update.wait()
-        eq_(data, [None])
-        update.clear()
-
-        self.expire_session()
-        eq_(update.is_set(), False)
-        self.client.retry(self.client.set, self.path, b'fred')
-        update.wait()
-        eq_(data[0], None)
-
     def test_func_stops(self):
         update = threading.Event()
         data = [True]
 
+        self.path += "f"
+
         fail_through = []
 
         @self.client.DataWatch(self.path)
@@ -144,50 +118,19 @@ class KazooDataWatcherTests(KazooTestCase):
             if fail_through:
                 return False
 
-        update.wait()
-        eq_(data, [b""])
-        update.clear()
-
-        fail_through.append(True)
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], b'fred')
-        update.clear()
-
-        self.client.set(self.path, b'asdfasdf')
-        update.wait(0.2)
-        eq_(data[0], b'fred')
-
-        d, stat = self.client.get(self.path)
-        eq_(d, b'asdfasdf')
-
-    def test_func_stops2(self):
-        update = threading.Event()
-        data = [True]
-
-        fail_through = []
-
-        @self.client.DataWatch(self.path, allow_missing_node=True)
-        def changed(d, stat):
-            data.pop()
-            data.append(d)
-            update.set()
-            if fail_through:
-                return False
-
-        update.wait()
+        update.wait(10)
         eq_(data, [None])
         update.clear()
 
         fail_through.append(True)
-        self.client.set(self.path, b'fred')
-        update.wait()
-        eq_(data[0], None)
+        self.client.create(self.path, b'fred')
+        update.wait(10)
+        eq_(data[0], b'fred')
         update.clear()
 
         self.client.set(self.path, b'asdfasdf')
         update.wait(0.2)
-        eq_(data[0], None)
+        eq_(data[0], b'fred')
 
         d, stat = self.client.get(self.path)
         eq_(d, b'asdfasdf')
@@ -201,16 +144,7 @@ class KazooDataWatcherTests(KazooTestCase):
 
         eq_(args, [None, None])
 
-    def test_no_such_node2(self):
-        args = []
-
-        @self.client.DataWatch("/some/path", allow_missing_node=True)
-        def changed(d, stat):
-            args.extend([d, stat])
-
-        eq_(args, [None, None])
-
-    def test_bad_watch_func(self):
+    def test_bad_watch_func2(self):
         counter = 0
 
         @self.client.DataWatch(self.path)
@@ -223,18 +157,63 @@ class KazooDataWatcherTests(KazooTestCase):
         counter += 1
         self.client.set(self.path, b'asdfasdf')
 
-    def test_bad_watch_func2(self):
-        counter = 0
+    def test_watcher_evaluating_to_false(self):
+        class WeirdWatcher(list):
+            def __call__(self, *args):
+                self.called = True
+        watcher = WeirdWatcher()
+        self.client.DataWatch(self.path, watcher)
+        self.client.set(self.path, b'mwahaha')
+        self.assertTrue(watcher.called)
 
-        @self.client.DataWatch(self.path, allow_missing_node=True)
-        def changed(d, stat):
-            if counter > 0:
-                raise Exception("oops")
+    def test_watcher_repeat_delete(self):
+        a = []
+        ev = threading.Event()
 
-        raises(Exception)(changed)
+        self.client.delete(self.path)
 
-        counter += 1
-        self.client.set(self.path, b'asdfasdf')
+        @self.client.DataWatch(self.path)
+        def changed(val, stat):
+            a.append(val)
+            ev.set()
+
+        eq_(a, [None])
+        ev.wait(10)
+        ev.clear()
+        self.client.create(self.path, b'blah')
+        ev.wait(10)
+        eq_(ev.is_set(), True)
+        ev.clear()
+        eq_(a, [None, b'blah'])
+        self.client.delete(self.path)
+        ev.wait(10)
+        eq_(ev.is_set(), True)
+        ev.clear()
+        eq_(a, [None, b'blah', None])
+        self.client.create(self.path, b'blah')
+        ev.wait(10)
+        eq_(ev.is_set(), True)
+        ev.clear()
+        eq_(a, [None, b'blah', None, b'blah'])
+
+    def test_watcher_with_closing(self):
+        a = []
+        ev = threading.Event()
+
+        self.client.delete(self.path)
+
+        @self.client.DataWatch(self.path)
+        def changed(val, stat):
+            a.append(val)
+            ev.set()
+        eq_(a, [None])
+
+        b = False
+        try:
+            self.client.stop()
+        except:
+            b = True
+        eq_(b, False)
 
 
 class KazooChildrenWatcherTests(KazooTestCase):
@@ -254,19 +233,38 @@ class KazooChildrenWatcherTests(KazooTestCase):
             all_children.extend(children)
             update.set()
 
-        update.wait()
+        update.wait(10)
         eq_(all_children, [])
         update.clear()
 
         self.client.create(self.path + '/' + 'smith')
-        update.wait()
+        update.wait(10)
         eq_(all_children, ['smith'])
         update.clear()
 
         self.client.create(self.path + '/' + 'george')
-        update.wait()
+        update.wait(10)
         eq_(sorted(all_children), ['george', 'smith'])
 
+    def test_child_watcher_with_event(self):
+        update = threading.Event()
+        events = [True]
+
+        @self.client.ChildrenWatch(self.path, send_event=True)
+        def changed(children, event):
+            events.pop()
+            events.append(event)
+            update.set()
+
+        update.wait(10)
+        eq_(events, [None])
+        update.clear()
+
+        self.client.create(self.path + '/' + 'smith')
+        update.wait(10)
+        eq_(events[0].type, EventType.CHILD)
+        update.clear()
+
     def test_func_style_child_watcher(self):
         update = threading.Event()
         all_children = ['fred']
@@ -279,17 +277,17 @@ class KazooChildrenWatcherTests(KazooTestCase):
 
         self.client.ChildrenWatch(self.path, changed)
 
-        update.wait()
+        update.wait(10)
         eq_(all_children, [])
         update.clear()
 
         self.client.create(self.path + '/' + 'smith')
-        update.wait()
+        update.wait(10)
         eq_(all_children, ['smith'])
         update.clear()
 
         self.client.create(self.path + '/' + 'george')
-        update.wait()
+        update.wait(10)
         eq_(sorted(all_children), ['george', 'smith'])
 
     def test_func_stops(self):
@@ -307,13 +305,13 @@ class KazooChildrenWatcherTests(KazooTestCase):
             if fail_through:
                 return False
 
-        update.wait()
+        update.wait(10)
         eq_(all_children, [])
         update.clear()
 
         fail_through.append(True)
         self.client.create(self.path + '/' + 'smith')
-        update.wait()
+        update.wait(10)
         eq_(all_children, ['smith'])
         update.clear()
 
@@ -332,21 +330,19 @@ class KazooChildrenWatcherTests(KazooTestCase):
             all_children.extend(children)
             update.set()
 
-        update.wait()
+        update.wait(10)
         eq_(all_children, [])
         update.clear()
 
         self.client.create(self.path + '/' + 'smith')
-        update.wait()
+        update.wait(10)
         eq_(all_children, ['smith'])
         update.clear()
         self.expire_session()
 
-        eq_(update.is_set(), False)
-
         self.client.retry(self.client.create,
                           self.path + '/' + 'george')
-        update.wait()
+        update.wait(20)
         eq_(sorted(all_children), ['george', 'smith'])
 
     def test_child_stop_on_session_loss(self):
@@ -360,21 +356,19 @@ class KazooChildrenWatcherTests(KazooTestCase):
             all_children.extend(children)
             update.set()
 
-        update.wait()
+        update.wait(10)
         eq_(all_children, [])
         update.clear()
 
         self.client.create(self.path + '/' + 'smith')
-        update.wait()
+        update.wait(10)
         eq_(all_children, ['smith'])
         update.clear()
         self.expire_session()
 
-        eq_(update.is_set(), False)
-
         self.client.retry(self.client.create,
                           self.path + '/' + 'george')
-        update.wait(3)
+        update.wait(4)
         eq_(update.is_set(), False)
         eq_(all_children, ['smith'])
 
